@@ -20,6 +20,8 @@
 package org.apache.opendal;
 
 import java.nio.charset.StandardCharsets;
+import java.util.Arrays;
+import java.util.List;
 import java.util.Map;
 
 /**
@@ -37,7 +39,7 @@ public class BlockingOperator extends NativeObject {
      * and see what config options each service supports.
      *
      * @param schema the name of the underneath service to access data from.
-     * @param map a map of properties to construct the underneath operator.
+     * @param map    a map of properties to construct the underneath operator.
      */
     public static BlockingOperator of(String schema, Map<String, String> map) {
         try (final Operator operator = Operator.of(schema, map)) {
@@ -50,6 +52,15 @@ public class BlockingOperator extends NativeObject {
         this.info = info;
     }
 
+    /**
+     * @return the cloned blocking operator.
+     * @see Operator#duplicate()
+     */
+    public BlockingOperator duplicate() {
+        final long nativeHandle = duplicate(this.nativeHandle);
+        return new BlockingOperator(nativeHandle, this.info);
+    }
+
     public void write(String path, String content) {
         write(path, content.getBytes(StandardCharsets.UTF_8));
     }
@@ -67,7 +78,7 @@ public class BlockingOperator extends NativeObject {
     }
 
     public Metadata stat(String path) {
-        return new Metadata(stat(nativeHandle, path));
+        return stat(nativeHandle, path);
     }
 
     public void createDir(String path) {
@@ -78,18 +89,38 @@ public class BlockingOperator extends NativeObject {
         copy(nativeHandle, sourcePath, targetPath);
     }
 
+    public void rename(String sourcePath, String targetPath) {
+        rename(nativeHandle, sourcePath, targetPath);
+    }
+
+    public void removeAll(String path) {
+        removeAll(nativeHandle, path);
+    }
+
+    public List<Entry> list(String path) {
+        return Arrays.asList(list(nativeHandle, path));
+    }
+
     @Override
     protected native void disposeInternal(long handle);
 
+    private static native long duplicate(long nativeHandle);
+
     private static native void write(long nativeHandle, String path, byte[] content);
 
     private static native byte[] read(long nativeHandle, String path);
 
     private static native void delete(long nativeHandle, String path);
 
-    private static native long stat(long nativeHandle, String path);
+    private static native Metadata stat(long nativeHandle, String path);
 
     private static native long createDir(long nativeHandle, String path);
 
     private static native long copy(long nativeHandle, String sourcePath, String targetPath);
+
+    private static native long rename(long nativeHandle, String sourcePath, String targetPath);
+
+    private static native void removeAll(long nativeHandle, String path);
+
+    private static native Entry[] list(long nativeHandle, String path);
 }
