@@ -28,7 +28,7 @@
 //!
 //! ```txt
 //! impl Accessor for OssBackend {
-//!     type Writer = oio::TwoWaysWriter<
+//!     type Writer = raw::TwoWays<
 //!         oio::MultipartUploadWriter<OssWriter>,
 //!         oio::AppendObjectWriter<OssWriter>,
 //!     >;
@@ -44,39 +44,6 @@ use std::task::Poll;
 use crate::raw::*;
 use crate::*;
 
-/// TwoWaysWrite is used to implement [`Write`] based on two ways.
-///
-/// Users can wrap two different writers together.
-pub enum TwoWaysWriter<ONE: oio::Write, TWO: oio::Write> {
-    /// The first type for the [`TwoWaysWriter`].
-    One(ONE),
-    /// The second type for the [`TwoWaysWriter`].
-    Two(TWO),
-}
-
-impl<ONE: oio::Write, TWO: oio::Write> oio::Write for TwoWaysWriter<ONE, TWO> {
-    fn poll_write(&mut self, cx: &mut Context<'_>, bs: &dyn oio::WriteBuf) -> Poll<Result<usize>> {
-        match self {
-            Self::One(one) => one.poll_write(cx, bs),
-            Self::Two(two) => two.poll_write(cx, bs),
-        }
-    }
-
-    fn poll_close(&mut self, cx: &mut Context<'_>) -> Poll<Result<()>> {
-        match self {
-            Self::One(one) => one.poll_close(cx),
-            Self::Two(two) => two.poll_close(cx),
-        }
-    }
-
-    fn poll_abort(&mut self, cx: &mut Context<'_>) -> Poll<Result<()>> {
-        match self {
-            Self::One(one) => one.poll_abort(cx),
-            Self::Two(two) => two.poll_abort(cx),
-        }
-    }
-}
-
 /// ThreeWaysWriter is used to implement [`Write`] based on three ways.
 ///
 /// Users can wrap three different writers together.
