@@ -69,6 +69,8 @@ pub use ftp::FtpConfig;
 mod gcs;
 #[cfg(feature = "services-gcs")]
 pub use gcs::Gcs;
+#[cfg(feature = "services-gcs")]
+pub use gcs::GcsConfig;
 
 #[cfg(feature = "services-ghac")]
 mod ghac;
@@ -122,6 +124,8 @@ pub use libsql::LibsqlConfig;
 mod memcached;
 #[cfg(feature = "services-memcached")]
 pub use memcached::Memcached;
+#[cfg(feature = "services-memcached")]
+pub use memcached::MemcachedConfig;
 
 #[cfg(feature = "services-memory")]
 mod memory;
@@ -171,6 +175,8 @@ pub use self::redis::RedisConfig;
 mod rocksdb;
 #[cfg(feature = "services-rocksdb")]
 pub use self::rocksdb::Rocksdb;
+#[cfg(feature = "services-rocksdb")]
+pub use self::rocksdb::RocksdbConfig;
 
 #[cfg(feature = "services-s3")]
 mod s3;
@@ -200,6 +206,8 @@ pub use supabase::Supabase;
 mod webdav;
 #[cfg(feature = "services-webdav")]
 pub use webdav::Webdav;
+#[cfg(feature = "services-webdav")]
+pub use webdav::WebdavConfig;
 
 #[cfg(feature = "services-webhdfs")]
 mod webhdfs;
@@ -254,6 +262,8 @@ pub use self::postgresql::PostgresqlConfig;
 mod atomicserver;
 #[cfg(feature = "services-atomicserver")]
 pub use self::atomicserver::Atomicserver;
+#[cfg(feature = "services-atomicserver")]
+pub use self::atomicserver::AtomicserverConfig;
 
 #[cfg(feature = "services-mysql")]
 mod mysql;
@@ -321,3 +331,10 @@ mod upyun;
 pub use upyun::Upyun;
 #[cfg(feature = "services-upyun")]
 pub use upyun::UpyunConfig;
+
+#[cfg(feature = "services-chainsafe")]
+mod chainsafe;
+#[cfg(feature = "services-chainsafe")]
+pub use chainsafe::Chainsafe;
+#[cfg(feature = "services-chainsafe")]
+pub use chainsafe::ChainsafeConfig;
